﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace KC.Cache
{
    /// <summary>
    /// 缓存基础接口
    /// </summary>
    public interface ICache
    {
        /// <summary>
        /// 写入缓存，单体
        /// </summary>
        /// <param name="value">对象数据</param>
        /// <param name="cacheKey">键</param>
        /// <param name="seconds">几秒过期</param>
        void WriteCache<T>(string cacheKey, T value, int seconds = 0) where T : class;



        /// <summary>
        /// 读取缓存，单体
        /// </summary>
        /// <param name="cacheKey">键</param>
        /// <returns></returns>
        T GetCache<T>(string cacheKey) where T : class;



        /// <summary>
        ///  获取所有key
        /// </summary>
        /// <returns></returns>
        List<string> GetAllKeys();

        /// <summary>
        /// 确定当前Key是否过期
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        bool HasExpire(string key);

        /// <summary>
        /// 移除指定数据缓存
        /// </summary>
        /// <param name="cacheKey">键</param>
        void RemoveCache(string key);

        /// <summary>
        /// 批量删除数据缓存
        /// </summary>
        /// <param name="keys"></param>
        void RemoveCache(List<string> keys);
    }
}
