﻿using System;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Reflection;

namespace Common
{
    public sealed class EntityAttributeHelper
    {
        /// <summary>
        ///  获取实体对象表名
        /// </summary>
        /// <returns></returns>
        public static string GetEntityTable<T>()
        {
            Type objTye = typeof(T);
            string entityName = "";
            var tableAttribute = objTye.GetCustomAttributes(true).OfType<TableAttribute>();
            var descriptionAttributes = tableAttribute as TableAttribute[] ?? tableAttribute.ToArray();

            entityName = descriptionAttributes.Any() ? descriptionAttributes.ToList()[0].Name : objTye.Name;
            return entityName;
        }

        /// <summary>
        ///  获取实体对象Key
        /// </summary>
        /// <returns></returns>
        public static string GetEntityKey<T>()
        {
            Type objTye = typeof(T);
            string entityName = "";
            var tableAttribute = objTye.GetCustomAttributes(true).OfType<Key>();
            var descriptionAttributes = tableAttribute as Key[] ?? tableAttribute.ToArray();

            entityName = descriptionAttributes.Any() ? descriptionAttributes.ToList()[0].Name : objTye.Name;
            return entityName;
        }

        /// <summary>
        ///  获取ApiAction值
        /// </summary>
        /// <returns></returns>
        public static string GetMark<T>()
        {
            Type objTye = typeof(T);
            string entityName = "";
            var tableAttribute = objTye.GetCustomAttributes(true).OfType<MarkAttribute>();
            var descriptionAttributes = tableAttribute as MarkAttribute[] ?? tableAttribute.ToArray();

            entityName = descriptionAttributes.Any() ? descriptionAttributes.ToList()[0].Name : objTye.Name;
            return entityName;
        }

        /// <summary>
        /// 获取访问元素
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static PropertyInfo[] GetProperties(Type type)
        {
            return type.GetProperties(BindingFlags.Public | BindingFlags.Instance);
        }

        /// <summary>
        ///  获取实体对象表名
        /// </summary>
        /// <returns></returns>
        public static string GetCalssTable<T>()
        {
            Type objTye = typeof(T);
            if (objTye.Name.Contains("List"))
            {
                objTye = objTye.GenericTypeArguments[0];
            }
            string entityName = "";
            try
            {
                var tableAttribute = objTye.GetCustomAttributes(true).OfType<TableAttribute>();
                var descriptionAttributes = tableAttribute as TableAttribute[] ?? tableAttribute.ToArray();
                entityName = descriptionAttributes.Any() ? descriptionAttributes.ToList()[0].Name : objTye.Name;
            }
            catch (Exception e)
            {
            }
            return entityName+"_";
        }
       
    }
}
