﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Common
{
    public sealed class ActionHelper
    {
        /// <summary>
        /// 获取一等奖、二等奖
        /// </summary>
        /// <param name="winbonus"></param>
        /// <returns></returns>
        public static List<WinbonusModel> GetWinbonus(string winbonus)
        {
            List<WinbonusModel> list = new List<WinbonusModel>();
            list.Add(new WinbonusModel() { item = "一等奖", wincount = "0", winmoney = "--" });
            list.Add(new WinbonusModel() { item = "二等奖", wincount = "0", winmoney = "--" });
            try
            {
                if (!string.IsNullOrEmpty(winbonus))
                {
                    var obj = JsonConvert.DeserializeObject<List<WinbonusModel>>(winbonus);
                    if (obj == null)
                        return list;
                    if (obj.Count != 2)
                        return list;
                    return obj;
                }
                return list;
            }
            catch (Exception e)
            {
                return list;
            }
        }
        /// <summary>
        /// 获取和值
        /// </summary>
        /// <param name="opencode"></param>
        /// <returns></returns>
        public static int GetHz(string opencode)
        {
            if (string.IsNullOrEmpty(opencode))
                return 0;
            try
            {
                var number = Array.ConvertAll(opencode.Split(','), s => int.Parse(s));
                return number.Sum();
            }
            catch (Exception e)
            {
                return 0;
            }
        }
        /// <summary>
        /// 获取大小比
        /// </summary>
        /// <param name="opencode"></param>
        /// <returns></returns>
        public static string GetDxb(string opencode,int number)
        {
            if (string.IsNullOrEmpty(opencode))
                return "--:--";
            try
            {
                var numbers = Array.ConvertAll(opencode.Split(','), s => int.Parse(s));
                int dnum = 0,xnum=0;
                foreach (var item in numbers)
                {
                    if (item > number)
                        dnum++;
                    else
                        xnum++;
                }
                return dnum+":"+xnum;
            }
            catch (Exception e)
            {
                return "--:--";
            }
        }

        /// <summary>
        /// 获取三区间比
        /// </summary>
        /// <param name="opencode"></param>
        /// <returns></returns>
        public static string GetQj3b(string opencode, int number)
        {
            if (string.IsNullOrEmpty(opencode))
                return "--:--:--";
            try
            {
                int q1 = number / 3 * 1, q2 = number / 3 * 2, q3 = number; 
                var numbers = Array.ConvertAll(opencode.Split(','), s => int.Parse(s));
                int q1num = 0, q2num = 0, q3num = 0;
                foreach (var item in numbers)
                {
                    if (item <= q1)
                        q1num++;
                    else if (item < q2)
                        q2num++;
                    else
                        q3num++;
                }
                return q1num + ":" + q2num + ":" + q3num;
            }
            catch (Exception e)
            {
                return "--:--:--";
            }
        }
    }
}
