﻿using Common.Models;
using Services.DAL;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace Services.BLL
{
    public class CommonBLL
    { 
        private CommonDAL dal = new CommonDAL();

        #region 查询

       
        /// <summary>
        /// 分页查询-Lambda
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="condition"></param>
        /// <param name="pagination"></param>
        /// <returns></returns>
        public PaginationDTO<IEnumerable<T>> FindPageList<T>(Expression<Func<T, bool>> condition, PaginationQuery pagination) where T : class, new()
        {
            return dal.FindPageList<T>(condition, pagination);
        }

        /// <summary>
        /// 集合查询（Lambda）
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="condition"></param>
        /// <returns></returns>
        public IEnumerable<T> FindList<T>(Expression<Func<T, bool>> condition) where T : class, new()
        {
            return dal.FindList<T>(condition);
        }

        /// <summary>
        /// 集合查询（Lambda）
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="condition"></param>
        /// <returns></returns>
        public T FindEntity<T>(Expression<Func<T, bool>> condition) where T : class, new()
        {
            return dal.FindEntity<T>(condition);
        }
        
        /// <summary>
        /// 数据条数
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="condition"></param>
        /// <returns></returns>
        public int Count<T>(Expression<Func<T, bool>> condition) where T : class, new()
        {
            return dal.Count<T>(condition);
        }

        #endregion

        #region sql查询

        /// <summary>
        /// sql 分页集合查询
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="condition"></param>
        /// <returns></returns>
        public PaginationDTO<IEnumerable<T>> FindPageList<T>(string sql, PaginationQuery pagination, object para = null) where T : class, new()
        {
            return dal.FindPageList<T>(sql, pagination, para);
        }
        /// <summary>
        /// sql查询
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="condition"></param>
        /// <returns></returns>
        public T FindEntity<T>(string sql, object para = null) where T : class, new()
        {
            return dal.FindEntity<T>(sql, para);
        }
        /// <summary>
        /// sql 数据集查询
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="sql"></param>
        /// <param name="para"></param>
        /// <returns></returns>
        public IEnumerable<T> FindList<T>(string sql, object para = null) where T : class, new()
        {
            return dal.FindList<T>(sql, para);
        }
        #endregion

        #region 编辑


        /// <summary>
        /// 实体新增
        /// </summary>
        /// <param name="sql">sql语句</param>
        /// <param name="para">参数化</param>
        public int Insert<T>(T model) where T : class, new()
        {
            return dal.Insert<T>(model);
        }

        /// <summary>
        /// 批量实体新增
        /// </summary>
        /// <param name="sql">sql语句</param>
        /// <param name="para">参数化</param>
        public int Insert<T>(List<T> list) where T : class, new()
        {
            return dal.Insert<T>(list);
        }

        /// <summary>
        /// 修改
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="model"></param>
        /// <returns></returns>
        public int Update<T>(T model) where T : class, new()
        {
            return dal.Update<T>(model);
        }

        #endregion

        #region 删除
        /// <summary>
        /// 删除
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="model"></param>
        /// <returns></returns>
        public int Delete<T>(T model) where T : class, new()
        {
            return dal.Delete<T>(model);
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="model"></param>
        /// <returns></returns>
        public int DeleteById<T>(int id) where T : class, new()
        {
            return dal.Delete<T>(id);
        }
        #endregion

    }
}
